package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.log.ResultListener;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.UserAndSliceApiWrapper;
import be.iminds.ilabt.jfed.util.common.GeniUrn;
import be.iminds.ilabt.jfed.highlevel.util.JavaFXLogger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.time.Instant;
import java.util.*;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 15:21
 */
public final class RegisterOrUpdateSliversTask extends Task {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(RegisterOrUpdateSliversTask.class);

    public enum Action {
        UPDATE_SLIVER_INFO,
        CREATE_SLIVER_INFO,
    }

    @Nonnull
    private final Action action;
    @Nonnull
    private final Slice slice;
    @Nonnull
    private final Collection<GeniUrn> createdSliverUrns;
    @Nullable
    private final Task extraTaskToDependOn;
    @Nonnull
    private final Instant sliverExpirationDate;
    @Nonnull
    private final Instant sliverCreationDate;
    @Nonnull
    private final Server auth;
    @Nonnull
    private final ResultListener[] extraResultListeners;

    private final UserAndSliceApiWrapper userAndSliceApiWrapper;
    private final JavaFXLogger logger;
    private final HighLevelTaskFactory highLevelTaskFactory;

    public RegisterOrUpdateSliversTask(@Nonnull Action action,
                                       @Nonnull Slice slice,
                                       @Nonnull Server auth,
                                       @Nonnull Collection<GeniUrn> createdSliverUrns,
                                       @Nonnull Instant sliverExpirationDate,
                                       @Nullable Task extraTaskToDependOn,
                                       @Nonnull UserAndSliceApiWrapper userAndSliceApiWrapper,
                                       @Nonnull JavaFXLogger logger,
                                       @Nonnull HighLevelTaskFactory highLevelTaskFactory,
                                       @Nonnull ResultListener... extraResultListeners) {
        super((action == Action.CREATE_SLIVER_INFO ? "Registering" : "Updating" )+
                " slivers created @ " + auth.getDefaultComponentManagerUrn());

        this.action = action;
        this.slice = slice;
        this.createdSliverUrns = createdSliverUrns;
        this.sliverExpirationDate = sliverExpirationDate;
        this.sliverCreationDate = Instant.now();
        this.auth = auth;
        this.extraTaskToDependOn = extraTaskToDependOn;

        this.userAndSliceApiWrapper = userAndSliceApiWrapper;
        this.logger = logger;
        this.highLevelTaskFactory = highLevelTaskFactory;
        this.extraResultListeners = extraResultListeners;
    }

    @Override
    protected void doTask(@Nonnull TaskExecution taskExecution) throws JFedException, InterruptedException {
        ResultListener[] resultListeners = Arrays.copyOf(extraResultListeners, extraResultListeners.length + 1);
        resultListeners[extraResultListeners.length] = taskExecution;
        JavaFXLogger logger = JavaFXLogger.wrap(this.logger, resultListeners);

        Collection<GeniUrn> sliverUrns = createdSliverUrns;

        LOG.debug("RegisterSliversTask has " + sliverUrns.size() + " sliver Urns to register.");
        if (!sliverUrns.isEmpty()) {
            if (action == Action.CREATE_SLIVER_INFO) {
                userAndSliceApiWrapper.registerAggregatesForSlice(logger,
                        slice.getCredentials(), slice.getUrn(),
                        auth.getDefaultComponentManagerAsGeniUrn(), sliverUrns,
                        Date.from(sliverExpirationDate),
                        Date.from(sliverCreationDate));
            }
            if (action == Action.UPDATE_SLIVER_INFO) {
                userAndSliceApiWrapper.updateSliverExpirationDateInfoForSlice(logger,
                        slice.getCredentials(), slice.getUrn(),
                        auth.getDefaultComponentManagerAsGeniUrn(), sliverUrns,
                        Date.from(sliverExpirationDate));
            }
        }
    }

    @Nonnull
    @Override
    public List<Task> initDependsOn() {
        List<Task> res = new ArrayList<>();
        if (extraTaskToDependOn != null)
            res.add(extraTaskToDependOn);
        res.add(highLevelTaskFactory.getSliceCredential(slice));
        return res;
    }
}
